package javax.swing;

import java.awt.Container;
import javax.accessibility.AccessibleRole;
import java.beans.ConstructorProperties;
import javax.accessibility.AccessibleContext;
import java.awt.Graphics;
import java.awt.AWTError;
import java.awt.LayoutManager;
import java.awt.Component;
import java.awt.Dimension;
import javax.accessibility.Accessible;

public class Box extends JComponent implements Accessible
{
    public Box(int n) {
        super.setLayout(new BoxLayout(this, n));
    }
    
    public static Box createHorizontalBox() {
        return new Box(0);
    }
    
    public static Box createVerticalBox() {
        return new Box(1);
    }
    
    public static Component createRigidArea(Dimension dimension) {
        return new Filler(dimension, dimension, dimension);
    }
    
    public static Component createHorizontalStrut(int width) {
        return new Box.Filler(new Dimension(width, 0), new Dimension(width, 0), new Dimension(width, 32767));
    }
    
    public static Component createVerticalStrut(int height) {
        return new Box.Filler(new Dimension(0, height), new Dimension(0, height), new Dimension(32767, height));
    }
    
    public static Component createGlue() {
        return new Box.Filler(new Dimension(0, 0), new Dimension(0, 0), new Dimension(32767, 32767));
    }
    
    public static Component createHorizontalGlue() {
        return new Box.Filler(new Dimension(0, 0), new Dimension(0, 0), new Dimension(32767, 0));
    }
    
    public static Component createVerticalGlue() {
        return new Box.Filler(new Dimension(0, 0), new Dimension(0, 0), new Dimension(0, 32767));
    }
    
    @Override
    public void setLayout(LayoutManager layoutManager) {
        throw new AWTError("Illegal request");
    }
    
    @Override
    protected void paintComponent(Graphics graphics) {
        if (this.ui != null) {
            super.paintComponent(graphics);
        }
        else if (this.isOpaque()) {
            graphics.setColor(this.getBackground());
            graphics.fillRect(0, 0, this.getWidth(), this.getHeight());
        }
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new Box.AccessibleBox();
        }
        return this.accessibleContext;
    }
    
    public static class Filler extends JComponent implements Accessible
    {
        @ConstructorProperties({ "minimumSize", "preferredSize", "maximumSize" })
        public Filler(Dimension minimumSize, Dimension preferredSize, Dimension maximumSize) {
            this.setMinimumSize(minimumSize);
            this.setPreferredSize(preferredSize);
            this.setMaximumSize(maximumSize);
        }
        
        public void changeShape(Dimension minimumSize, Dimension preferredSize, Dimension maximumSize) {
            this.setMinimumSize(minimumSize);
            this.setPreferredSize(preferredSize);
            this.setMaximumSize(maximumSize);
            this.revalidate();
        }
        
        @Override
        protected void paintComponent(Graphics graphics) {
            if (this.ui != null) {
                super.paintComponent(graphics);
            }
            else if (this.isOpaque()) {
                graphics.setColor(this.getBackground());
                graphics.fillRect(0, 0, this.getWidth(), this.getHeight());
            }
        }
        
        @Override
        public AccessibleContext getAccessibleContext() {
            if (this.accessibleContext == null) {
                this.accessibleContext = new Box.Filler.AccessibleBoxFiller();
            }
            return this.accessibleContext;
        }
        
        protected class AccessibleBoxFiller extends Component.AccessibleAWTComponent
        {
            @Override
            public AccessibleRole getAccessibleRole() {
                return AccessibleRole.FILLER;
            }
        }
    }
    
    protected class AccessibleBox extends Container.AccessibleAWTContainer
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.FILLER;
        }
    }
}
